#include <chrono>
#include <iostream>
#include <ros/ros.h>
#include <stdio.h>

#include "std_msgs/String.h"

// google glog
#include <glog/logging.h>

#include <unistd.h>

#include "receiver.h"

using namespace std;

namespace test_pub {

class ReceiverPrivate {
public:
  ReceiverPrivate() { LOG(INFO) << "in ReceiverPrivate"; };

  ReceiverPrivate(ros::NodeHandle &nh) {
    this->nh_ = nh;
    seq_ = 0;
  };

  ~ReceiverPrivate() { LOG(INFO) << "in ~ReceiverPrivate"; };

public:
  ros::NodeHandle nh_;
  ros::Publisher pub_;
  ros::Subscriber sub_;
  size_t seq_;
};

Receiver::Receiver() { LOG(INFO) << "in Receiver()..."; }

Receiver::Receiver(ros::NodeHandle &nh) {
  LOG(INFO) << "in Receiver()...";

  // construct ReceiverPrivate object
  m_pHandle = make_shared<ReceiverPrivate>(nh);

  // pub
  m_pHandle->pub_ = nh.advertise<std_msgs::String> ("out", 1);

  // sub
  m_pHandle->sub_ = nh.subscribe("in", 1, &Receiver::cbHandleReaderOutMsg, this);
}

Receiver::~Receiver() {
  LOG(INFO) << "in ~Receiver()...";
}

/* ROS callback functions */
void Receiver::cbHandleReaderOutMsg(const std_msgs::String msg) {
  LOG(INFO) << "in Receiver::cbHandleReaderOutMsg...";
  m_pHandle->seq_ ++;
  m_pHandle->pub_.publish(msg);
  LOG(INFO) << "Receiver callback end...";
}

}